#include <chronoptics/tof/csf_writer.hpp>
#include <chronoptics/tof/gige_interface.hpp>
#include <iostream>

namespace tof = chronoptics::tof;

int main() {
  /* Number of frames to capture */
  const int numFrames = 10;

  /* Number of frames to drop before capturing */
  const int dropFrames = 100;

  /* Filename to save to */
  const char* filename = "frames.csf";

  /* Serial number of the camera to connect to. If this is empty, the
     first camera that is found will be connected to */
  const char* serial = "2020004";

  tof::GigeInterface cam;
  tof::DiscoveryMessage msg = cam.find(serial);
  cam.connect(msg);

  if (!cam.test_stream_packet_size()) {
    std::cout << "Lower stream packet size\n";
    return -1;
  }

  tof::CsfWriter writer = tof::create_csf_writer_gige_interface(filename, cam);

  /* Start streaming raw frames from the camera */
  cam.start_depth_stream(tof::DepthStreamType::RAW);

  /* Drop frames */
  for (int i = 0; i < dropFrames; i++) {
    cam.get_depth_data();
  }

  std::cout << "Frames dropped" << std::endl;

  /* Capture frames */
  for (int i = 0; i < numFrames; i++) {
    tof::Data frame = cam.get_depth_data();
    writer.write_frame(frame);
  }

  std::cout << "Frames captured" << std::endl;
  cam.stop_depth_stream();
  cam.disconnect();
}
