# -*- coding: utf-8 -*-
"""
Created on Fri Mar 12 16:15:47 2021

@author: RefaelWhyte
"""

from typing import List
import chronoptics.tof as tof
import matplotlib.pyplot as plt
import numpy as np


def handle_close(evt):
    cam.stop()
    return


def getFrame(frames: List[tof.Data], frame_type: tof.FrameType):
    for frame in frames:
        if frame.frameType() == frame_type:
            return frame
    return None


serial = "202001a"


cam = tof.KeaCamera(tof.ProcessingConfig(), serial)

config = cam.getCameraConfig()
config.reset()

config.setModulationFrequency(0, 80.0)
config.setPhaseShifts(0, [0.0, 0.25, 0.375, 0.625])
config.setIntegrationTime(0, [300, 300, 300, 300])
config.setDutyCycle(0, 0.4)
config.setFlip(0, True)

config.addFrame()
config.setModulationFrequency(1, 100.0)
config.setPhaseShifts(1, [0.0, 0.25, 0.375, 0.625])
config.setIntegrationTime(1, [300, 300, 300, 300])
config.setDutyCycle(1, 0.4)
config.setFlip(1, True)

cam.setCameraConfig(config)

proc_config = tof.ProcessingConfig()
proc_config.setTemporalEnabled(True)
proc_config.setMedianEnabled(True)
proc_config.setPhaseUnwrappingEnabled(True)
proc_config.setPhaseUnwrappingMaxOffset(np.pi / 2)

cam.setProcessConfig(proc_config)


roi = config.getRoi(0)
irows = roi.getImgRows()
icols = roi.getImgCols()

# Start RGB-D streaming.
tof.selectStreams(
    cam, [tof.FrameType.BGR, tof.FrameType.RADIAL, tof.FrameType.INTENSITY]
)

cam.start()

fig = plt.figure()
fig.canvas.mpl_connect("close_event", handle_close)
ax0 = plt.subplot(131)
ax1 = plt.subplot(132)
ax2 = plt.subplot(133)

nrows = 1920
ncols = 1080
nchs = 3

bgrimg = ax0.imshow(
    np.zeros((ncols, nrows, nchs), dtype=np.uint8),
    vmin=0,
    vmax=255,
    interpolation="none",
)
ax0.set_title("Colour")
iimg = ax1.imshow(
    np.zeros((irows, icols), dtype=np.uint8),
    vmin=0,
    vmax=255,
    interpolation="none",
    cmap="gray",
)
ax1.set_title("Intensity")
rimg = ax2.imshow(
    np.zeros((irows, icols), dtype=np.uint16),
    vmin=0,
    vmax=65535,
    interpolation="none",
    cmap="jet",
)
ax2.set_title("Distance")

while cam.isStreaming():
    frames = cam.getFrames()
    # It comes out as a BGR, but matplotlib displays as an RGB.
    bgr_frame = getFrame(frames, tof.FrameType.BGR)
    i_frame = getFrame(frames, tof.FrameType.INTENSITY)
    r_frame = getFrame(frames, tof.FrameType.RADIAL)

    bgrimg.set_data((np.copy(np.flipud(np.flip(np.asarray(bgr_frame), axis=2)))))
    iimg.set_data(np.copy(np.asarray(i_frame)))
    rimg.set_data(np.copy(np.asarray(r_frame)))
    plt.draw()
    plt.show(block=False)
    # This makes sure the event loop is updated
    plt.pause(0.1)
